SystemOrganization addCategory: #'Generic-Spreadsheet'!
SystemOrganization addCategory: #'Generic-Spreadsheet-Tests'!

Object subclass: #GenericCell
	instanceVariableNames: 'cellValue cellType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Generic-Spreadsheet'!

!GenericCell commentStamp: 'SauravBhattacharya 7/17/2012 09:36' prior: 0!
A GenericCell is a class for a single cell of a GenericSpreadsheet .

Instance Variables
	cellType:		<Object>
	cellValue:		<Object>

cellType
	- Type of the cell of the following:
		1) String
		2) Numeric
		3) Date

cellValue
	- Value displayed by the cell.
!

!GenericCell methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/16/2012 07:27'!
cellType
	^cellType .! !

!GenericCell methodsFor: 'private' stamp: 'SauravBhattacharya 7/16/2012 07:27'!
cellType:aCellType
	cellType := aCellType .! !

!GenericCell methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/16/2012 07:24'!
cellValue
	^cellValue! !

!GenericCell methodsFor: 'private' stamp: 'SauravBhattacharya 7/16/2012 07:25'!
cellValue:aCellValue
	cellValue:= aCellValue .! !

!GenericCell methodsFor: 'initialize-release' stamp: 'SauravBhattacharya 7/16/2012 08:20'!
initialize 
	super initialize.
	cellValue:= '_'.
	cellType:= 'string'. "string/numeric/dates"! !

Object subclass: #GenericSpreadsheet
	instanceVariableNames: 'spreadsheetTitle aCollectionOfWorksheets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Generic-Spreadsheet'!

!GenericSpreadsheet commentStamp: 'SauravBhattacharya 7/17/2012 09:42' prior: 0!
Class intention
	I am GenericSpreadsheet class, for Spreadsheet support in Pharo Smalltalk.

Class collaborations
	Yuriy Mironenko and Carla Florencia.!

!GenericSpreadsheet methodsFor: 'adding' stamp: 'SauravBhattacharya 8/16/2012 00:39'!
addNewWorksheetWithTitle: aTitle 

	aCollectionOfWorksheets  add: GenericWorksheet new .	
	(aCollectionOfWorksheets at: aCollectionOfWorksheets size ) worksheetTitle: aTitle.! !

!GenericSpreadsheet methodsFor: 'initialize-release' stamp: 'SauravBhattacharya 7/18/2012 05:02'!
initialize 
	super initialize .
	spreadsheetTitle := 'Default Spreadsheet Title'.
	aCollectionOfWorksheets := OrderedCollection new.
	aCollectionOfWorksheets add: GenericWorksheet new.! !

!GenericSpreadsheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/18/2012 05:07'!
numberOfWorksheets
	^aCollectionOfWorksheets size.! !

!GenericSpreadsheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/24/2012 03:32'!
spreadsheetTitle
	^spreadsheetTitle.! !

!GenericSpreadsheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/24/2012 03:31'!
spreadsheetTitle: aByteString 
	spreadsheetTitle:= aByteString asString .! !

!GenericSpreadsheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/16/2012 07:08'!
worksheetWithIndex: anIndex
	^ aCollectionOfWorksheets at: anIndex.! !

Object subclass: #GenericWorksheet
	instanceVariableNames: 'maxNumOfRows maxNumOfColumns worksheetCells worksheetTitle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Generic-Spreadsheet'!

!GenericWorksheet commentStamp: 'SauravBhattacharya 7/17/2012 09:38' prior: 0!
A GenericWorksheet is a class for a single worksheet of a GenericSpreadsheet.

Instance Variables
	maxNumOfColumns:		<Object>
	maxNumOfRows:		<Object>
	myWorksheet:		<Object>
	worksheetTitle:		<Object>

maxNumOfColumns
	- Allowed columns in the worksheet.
	
maxNumOfRows
	- Allowed rows in the worksheet.

myWorksheet
	- Collection of cells in the Dictionary data structure

worksheetTitle
	- Title of  the worksheet.
!

!GenericWorksheet methodsFor: 'adding' stamp: 'SauravBhattacharya 7/24/2012 03:28'!
addNewCellWithRowIndex: i columnIndex: j withCellValue: aValue 

	| aCell aPoint |
	
	aCell:= GenericCell new.	
	aCell cellValue:aValue.
	
	(aValue isKindOf: Date) ifTrue: [aCell cellType: 'Date'].
	(aValue isKindOf: Number) ifTrue: [aCell cellType: 'Number'].
	(aValue isKindOf: String) ifTrue: [aCell cellType: 'String']. 
	
	i>maxNumOfRows ifTrue: [maxNumOfRows := i].
	j>maxNumOfColumns ifTrue: [maxNumOfColumns := j].
	
	aPoint := Point new.
	aPoint := i@j.	
	worksheetCells at:aPoint put:aCell.
	! !

!GenericWorksheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/19/2012 03:35'!
cellWithRowIndex:row columnIndex:column
	^worksheetCells  at:(row@column) ifAbsent:[^nil].
	
	 
	! !

!GenericWorksheet methodsFor: 'initialize-release' stamp: 'SauravBhattacharya 8/16/2012 00:10'!
initialize 
	"Worksheet of size 20*10 is generated"

	super initialize.
	worksheetCells  := Dictionary new.
	worksheetTitle:= 'Default Worksheet Title'.
	maxNumOfColumns := 1.
	maxNumOfRows :=1.
	
	! !

!GenericWorksheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/16/2012 07:18'!
maxNumOfColumns
	^maxNumOfColumns.! !

!GenericWorksheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/16/2012 07:18'!
maxNumOfRows
	^maxNumOfRows.! !

!GenericWorksheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/24/2012 03:34'!
sizeOfWorksheet
	^ maxNumOfRows@maxNumOfColumns .! !

!GenericWorksheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/16/2012 07:30'!
worksheetTitle
	^worksheetTitle.! !

!GenericWorksheet methodsFor: 'accessing' stamp: 'SauravBhattacharya 7/24/2012 03:11'!
worksheetTitle: aByteString 
	worksheetTitle:= aByteString asString .! !

TestCase subclass: #WorksheetTest
	instanceVariableNames: 'ss ws'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Generic-Spreadsheet-Tests'!

!WorksheetTest methodsFor: 'running' stamp: 'SauravBhattacharya 7/24/2012 02:39'!
setUp

	ss:= GenericSpreadsheet new.
	ws := ss worksheetWithIndex: 1.! !

!WorksheetTest methodsFor: 'testing' stamp: 'SauravBhattacharya 7/24/2012 03:07'!
testCellsReadingWriting
| endOfTheWorld|

	ws addNewCellWithRowIndex: 7 columnIndex: 12 withCellValue: 'aTest' .
	self assert: (ws cellWithRowIndex:7 columnIndex:12) cellValue= 'aTest'.

	ws addNewCellWithRowIndex: 7 columnIndex: 12 withCellValue: 666 .
	self assert: (ws cellWithRowIndex:7 columnIndex:12) cellValue= 666.

	endOfTheWorld:=Date year: 2012 month: 12 day: 21.	
	ws addNewCellWithRowIndex: 7 columnIndex: 12 withCellValue: endOfTheWorld .
	self assert: (ws cellWithRowIndex:7 columnIndex:12) cellValue= endOfTheWorld.
! !

!WorksheetTest methodsFor: 'testing' stamp: 'SauravBhattacharya 7/24/2012 03:14'!
testReadingFromEmptyTable
	self assert: (ws cellWithRowIndex:1 columnIndex:10) isNil.
! !

!WorksheetTest methodsFor: 'testing' stamp: 'SauravBhattacharya 7/24/2012 03:34'!
testResizingOnTableFillingByData

	self assert: ws sizeOfWorksheet  = (0@0).

	ws addNewCellWithRowIndex: 100 columnIndex: 2 withCellValue: 1.
	self assert: ws sizeOfWorksheet  = (100@2).

	ws addNewCellWithRowIndex: 2 columnIndex: 100 withCellValue: 2.
	self assert: ws sizeOfWorksheet  = (100@100).
! !

!WorksheetTest methodsFor: 'testing' stamp: 'SauravBhattacharya 7/24/2012 03:34'!
testSizeOfEmptyTable

	self assert: (ws maxNumOfColumns = 0).
	self assert: (ws maxNumOfRows = 0).
	self assert: (ws sizeOfWorksheet  = (0@0)).
! !

!WorksheetTest methodsFor: 'testing' stamp: 'SauravBhattacharya 7/24/2012 03:33'!
testTitle

	ss spreadsheetTitle:'Test Title'.
	self assert: ss spreadsheetTitle = 'Test Title'.
	
	ss spreadsheetTitle: 'Another'.
	self assert: ss spreadsheetTitle = 'Another'.
! !

!WorksheetTest methodsFor: 'testing' stamp: 'SauravBhattacharya 7/24/2012 03:33'!
testTitleToStringConversion

	ss spreadsheetTitle: (Date current).
	self assert: (ss spreadsheetTitle isKindOf: String).
	
! !
